#pragma once
#include <QMainWindow>
#include "BehaviorPerform.h"
#include <QGraphicsItem>
#include <QGraphicsView>

uint qHash(const std::shared_ptr<LogicalNode> data, uint seed) noexcept;

enum class AcceptType {
	NONE = 0, PREV_LEVEL, NEXT_LEVEL, PREV_SIBLING, NEXT_SIBLING
};

class BehaviorsPresent;
/// <summary>
/// ڵʾ
/// </summary>
class NodePresent : public QGraphicsItem {
private:
	BehaviorsPresent* const _widget_p;
	QVector<double>& _columns_width_seqs;
	std::shared_ptr<LogicalNode> _node_bind;

	AcceptType _drop_target = AcceptType::NONE;

public:
	static const double padding;
	NodePresent(BehaviorsPresent* pwidget, QVector<double>& columns_set, std::shared_ptr<LogicalNode> bind);
	/// <summary>
	/// 󶨵߼ڵ
	/// </summary>
	/// <returns></returns>
	std::shared_ptr<LogicalNode> logicalBind() const;

	QRectF contentMeasure() const;
	AcceptType testAccept(const QPointF& local_pos, const QString& kind_str) const;

	// ͨ QGraphicsItem ̳
	QRectF boundingRect() const override;
	void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

protected:
	virtual void dragEnterEvent(QGraphicsSceneDragDropEvent* event);
	virtual void dragLeaveEvent(QGraphicsSceneDragDropEvent* event);
	virtual void dragMoveEvent(QGraphicsSceneDragDropEvent* event);
	virtual void dropEvent(QGraphicsSceneDragDropEvent* event);

};

/// <summary>
/// ֧ͼδ
/// </summary>
class BranchPresent : public QGraphicsItem {
private:
	const QHash<std::shared_ptr<LogicalNode>, NodePresent*>& _present_set_bind;
	NodePresent* const _head_node;

	QPointF _arrow_start, _arrow_end;

public:
	BranchPresent(const QHash<std::shared_ptr<LogicalNode>, NodePresent*>& present_set, NodePresent* head_anchor);

	std::shared_ptr<LogicalNode> headNode() const;

	QRectF startOutline() const;
	QRectF endOutline() const;
	void resetArrow(const QPointF& start, const QPointF& end);

	// ͨ QGraphicsItem ̳
	QRectF boundingRect() const override;
	void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

};


/// <summary>
/// Ϊ༭չʾ
/// </summary>
class BehaviorsPresent : public QGraphicsView
{
	Q_OBJECT
private:
	QGraphicsScene _bind_scene;

	const double _space_h = 30;
	QVector<double> _column_aligns;
	QHash<std::shared_ptr<LogicalNode>, NodePresent*> _present_peers;
	QList<BranchPresent*> _branch_list;

	/// <summary>
	/// Ϊڵ
	/// </summary>
	std::shared_ptr<BehaviorMapNode> _bind_maproot = nullptr;

signals:
	void clickedUponAtItem(NodePresent *node);

public:
	uint64_t _node_id_max = 0;
	BehaviorsPresent(QWidget* parent = nullptr);

	/// <summary>
	/// 滻Ϊ
	/// </summary>
	/// <param name="root"></param>
	void setRoot(std::shared_ptr<BehaviorMapNode> root);

	/// <summary>
	/// ʾڵ
	/// </summary>
	/// <param name="_present_peers"></param>
	/// <param name="ins"></param>
	NodePresent* presentAllocate(std::shared_ptr<LogicalNode> ins);

	/// <summary>
	/// ²
	/// </summary>
	void relayout();
	/// <summary>
	/// ߴ
	/// </summary>
	/// <param name="ins"></param>
	/// <param name="_outline_occupy"></param>
	/// <returns></returns>
	QSizeF outlineMeasure(std::shared_ptr<LogicalNode> ins, QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy);
	/// <summary>
	/// ڵ㲼
	/// </summary>
	/// <param name="_outline_occupy">ߴ¼</param>
	/// <param name="ins">ָڵ</param>
	/// <param name="origin_offset">Ͻ</param>
	/// <returns>½</returns>
	QPointF nodeRelayout(QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy,
		std::shared_ptr<LogicalNode> ins, const QPointF& origin_offset);

protected:
	void mousePressEvent(QMouseEvent *e) override;
};


#include <QListView>
class NodeTypesView : public QListView {
public:
	NodeTypesView(QWidget* parent = nullptr);

protected:
	virtual void startDrag(Qt::DropActions supported) override;
};


#include <QStandardItemModel>
#include <QTextBrowser>
#include <MessageLoader.h>
#include <QStandardItemModel>
#include <QTableView>
#include <QStackedWidget>
#include <QLabel>
#include "BehaviorConfigurationPanel.h"
#include "CompareNodeConfiguration.h"
#include "ActionNodeConfiguration.h"
#include "ModifyNodeConfiguration.h"

/// <summary>
/// Ϊڵ༭
/// </summary>
class BehaviorEditor : public QMainWindow {
private:
	std::shared_ptr<MessageLoader> _global_loader;
	std::shared_ptr<MapKernel> _global_kernal;

	NodeTypesView* const _type_view;
	QStandardItemModel* const _type_model;
	BehaviorsPresent* const _logical_present;
	QTabWidget* const _message_panel;
	QStackedWidget* const _stacked_panel;
	BehaviorMapConfigurationPanel* const _map_configuration;
	CompareNodeConfiguration* const _compare_configuration;
	ActionNodeConfiguration* const _action_configuration;
	ModifyNodeConfiguration* const _modify_configuration;
	QLabel* const _default_configuration;

	std::shared_ptr<BehaviorMapNode> _map_root;
	QUrl _current_fileurl;

	void open_behavior_map();
	void new_behavior_map();

	void nodeTypesViewInit(QStandardItemModel* m);

	// ==============================================
	QTextBrowser* const _logs_present;

	void configurationPanel(NodePresent *n);
	
public:
	BehaviorEditor(QWidget* parent = nullptr);

};